==> hestia_earth_models-0.61.0-py3-none-any.whl: hestia_earth/models/cycle/animal/input/properties.py <==
"""
Input Properties

This model handles multiple cases:
- when the [impactAssessment](https://hestia.earth/schema/Input#impactAssessment) field is set, the model
adds properties to the `Input` when they are connected to another `Cycle` that has an indentical `Product`;
- when the `dryMatter` property is set, the model recalculates the other properties based on the data from feedipedia;
- when the `min` and `max` are set, the model averages the `value`.
"""
from hestia_earth.schema import SchemaType
from hestia_earth.utils.model import find_term_match

from hestia_earth.models.log import logShouldRun
from hestia_earth.models.utils import _load_calculated_node
from hestia_earth.models.utils.feedipedia import rescale_properties_from_dryMatter
from ...utils import should_run_properties_value, average_blank_node_properties_value
from ... import MODEL

REQUIREMENTS = {
    "Cycle": {
        "animals": [{
            "@type": "Animal",
            "inputs": [{
                "@type": "Input",
                "or": {
                    "impactAssessment": "",
                    "properties": [{"@type": "Property", "@id": "dryMatter"}]
                }
            }]
        }]
    }
}
RETURNS = {
    "Animal": [{
        "inputs": [{
            "@type": "Input",
            "properties": [{
                "@type": "Property"
            }]
        }]
    }]
}
MODEL_KEY = 'properties'


def _find_related_product(input: dict):
    impact = input.get('impactAssessment')
    impact = _load_calculated_node(impact, SchemaType.IMPACTASSESSMENT) if impact else {}
    cycle = impact.get('cycle') if impact else None
    cycle = _load_calculated_node(cycle, SchemaType.CYCLE) if cycle else None
    products = (cycle or {}).get('products', [])
    return find_term_match(products, input.get('term', {}).get('@id'))


def _run_input_by_impactAssessment(cycle: dict):
    def exec(input: dict):
        term_id = input.get('term', {}).get('@id')
        product = _find_related_product(input)
        properties = product.get('properties', [])
        all_properties = input.get('properties', [])
        new_properties = [p for p in properties if not find_term_match(all_properties, p.get('term', {}).get('@id'))]
        for prop in new_properties:
            logShouldRun(cycle, MODEL, term_id, True, property=prop.get('term', {}).get('@id'))
        return {**input, 'properties': all_properties + new_properties} if new_properties else input
    return exec


def _should_run_by_impactAssessment(input: dict):
    return bool(input.get('impactAssessment', None))


def _should_run_by_dryMatter(input: dict):
    return find_term_match(input.get('properties', []), 'dryMatter') is not None


def _run_animal(cycle: dict, animal: dict):
    # filter list of inputs to run
    inputs = [
        i for i in animal.get('inputs', []) if any([
            _should_run_by_impactAssessment(i),
            _should_run_by_dryMatter(i),
            should_run_properties_value(i)
        ])
    ]
    inputs = list(map(_run_input_by_impactAssessment(cycle), inputs))
    inputs = rescale_properties_from_dryMatter(MODEL, cycle, inputs)
    inputs = average_blank_node_properties_value(cycle, inputs)
    return animal | {'inputs': inputs}


def run(cycle: dict):
    return list(map(lambda a: _run_animal(cycle, a), cycle.get('animals', [])))

==> hestia_earth_models-0.64.3-py3-none-any.whl: hestia_earth/models/cycle/animal/input/properties.py <==
"""
Input Properties

This model handles multiple cases:
- when the [impactAssessment](https://hestia.earth/schema/Input#impactAssessment) field is set, the model
adds properties to the `Input` when they are connected to another `Cycle` that has an indentical `Product`;
- when the `dryMatter` property is set, the model recalculates the other properties based on the data from feedipedia;
- when the `min` and `max` are set, the model averages the `value`.
"""
from hestia_earth.schema import SchemaType
from hestia_earth.utils.model import find_term_match

from hestia_earth.models.log import logShouldRun
from hestia_earth.models.utils import _load_calculated_node
from hestia_earth.models.utils.blank_node import merge_blank_nodes
from hestia_earth.models.utils.feedipedia import rescale_properties_from_dryMatter
from ...utils import should_run_properties_value, average_blank_node_properties_value
from ... import MODEL

REQUIREMENTS = {
    "Cycle": {
        "animals": [{
            "@type": "Animal",
            "inputs": [{
                "@type": "Input",
                "or": {
                    "impactAssessment": "",
                    "properties": [{"@type": "Property", "@id": "dryMatter"}]
                }
            }]
        }]
    }
}
RETURNS = {
    "Animal": [{
        "inputs": [{
            "@type": "Input",
            "properties": [{
                "@type": "Property"
            }]
        }]
    }]
}
MODEL_KEY = 'properties'


def _find_related_product(input: dict):
    impact = input.get('impactAssessment')
    impact = _load_calculated_node(impact, SchemaType.IMPACTASSESSMENT) if impact else {}
    cycle = impact.get('cycle') if impact else None
    cycle = _load_calculated_node(cycle, SchemaType.CYCLE) if cycle else None
    products = (cycle or {}).get('products', [])
    return find_term_match(products, input.get('term', {}).get('@id'))


def _run_input_by_impactAssessment(cycle: dict):
    def exec(input: dict):
        term_id = input.get('term', {}).get('@id')
        product = _find_related_product(input)
        properties = product.get('properties', [])
        all_properties = input.get('properties', [])
        new_properties = [p for p in properties if not find_term_match(all_properties, p.get('term', {}).get('@id'))]
        for prop in new_properties:
            logShouldRun(cycle, MODEL, term_id, True, property=prop.get('term', {}).get('@id'))
        return {**input, 'properties': merge_blank_nodes(all_properties, new_properties)} if new_properties else input
    return exec


def _should_run_by_impactAssessment(input: dict):
    return bool(input.get('impactAssessment', None))


def _should_run_by_dryMatter(input: dict):
    return find_term_match(input.get('properties', []), 'dryMatter') is not None


def _run_animal(cycle: dict, animal: dict):
    # filter list of inputs to run
    inputs = [
        i for i in animal.get('inputs', []) if any([
            _should_run_by_impactAssessment(i),
            _should_run_by_dryMatter(i),
            should_run_properties_value(i)
        ])
    ]
    inputs = list(map(_run_input_by_impactAssessment(cycle), inputs))
    inputs = rescale_properties_from_dryMatter(MODEL, cycle, inputs)
    inputs = average_blank_node_properties_value(cycle, inputs)
    return animal | {'inputs': inputs}


def run(cycle: dict):
    return list(map(lambda a: _run_animal(cycle, a), cycle.get('animals', [])))
